/*
 * Customizable Origami - Whale - https://www.thingiverse.com/thing:2734803
 * by Dennis Hofmann - https://www.thingiverse.com/mightynozzle/about
 * created 2017-12-28
 * version v1.0
 *
 * Changelog
 * --------------
 * v1.0:
 *      - final design
 * --------------
 * 
 * This work is licensed under the Creative Commons - Attribution - NonCommercial - ShareAlike license.
 * https://creativecommons.org/licenses/by-nc-sa/3.0/
 */


 // Parameter Section //
//-------------------//

// preview[view:north, tilt:top]

// The maximum size of the longest side. The shorter side will be automatically resized in same ratio.
max_size_in_millimeter = 120; //[20:300]

// The height of the model.
model_height_in_millimeter = 5; //[0.2:0.2:100]

// Thickness of the outer outline.
outline_size_in_millimeter = 1.4; //[0.5:0.1:20]

// Thickness of the inner outline.
inline_size_in_millimeter = 0.7; //[0.5:0.1:20]

// Roundiness of the ends of the inner outlines.
inline_edge_radius_in_millimeter = 0.6; //[0.0:0.1:5]

// Flip model
flip_model = "no"; //[yes,no]

/*[hidden]*/
max_size = max_size_in_millimeter;
model_height = model_height_in_millimeter;
$fn=32;

// Outer Points
A1 = [870, 680];
A2 = [1298, 628];
A3 = [1458, 679];
A4 = [1575, 779];
A5 = [1658, 737];
A6 = [1722, 666];
A7 = [1839, 648];
A8 = [1796, 720];
A9 = [1931, 771];
A10 = [1796, 780];
A11 = [1764, 774];
A12 = [1670, 864];
A13 = [1307, 916];
A14 = [1301, 1009];
A15 = [1262, 1023];
A16 = [1188, 918];
A17 = [1009, 933];
A18 = [880, 817];

outline = [A1, A2, A3, A4, A5, A6, A7, A8, A9, A10, A11, A12, A13, A14, A15, A16, A17, A18];

// Inner Points
B1 = [1046, 820];
B2 = [1005, 789];
B3 = [1108, 792];
B4 = [1308, 844];
B5 = [1139, 846];
B6 = [1126, 825];

// Polygons
C1 = [A1, B2, A2];
C2 = [B2, A2, B4, B3, B1];
C3 = [B1, B3, B6];
C4 = [B3, B4, B5, B6];
C5 = [A1, B2, B1, B6, B5, A16, A17, A18];
C6 = [B5, B4, A13, A14, A15, A16];
C7 = [A2, A3, A4, A12, A13, B4];
C8 = [A4, A5, A11, A12];
C9 = [A5, A6, A7, A8];
C10 = [A5, A8, A9, A10, A11];

cut_polygons = [C1, C2, C3, C4, /*C5,*/ C6, C7, C8, C9, C10];

min_x = A1[0];
min_y = A2[1];
max_x = A9[0];
max_y = A15[1];

x_size = max_x - min_x;
y_size = max_y - min_y;

x_factor = max_size / x_size;
y_factor = max_size / y_size;

inline_size = x_size > y_size ? inline_size_in_millimeter / x_factor / 2: inline_size_in_millimeter / y_factor / 2;
inline_edge_radius = x_size > y_size ? inline_edge_radius_in_millimeter / x_factor: inline_edge_radius_in_millimeter / y_factor;
outline_size = x_size > y_size ? outline_size_in_millimeter / x_factor - inline_size: outline_size_in_millimeter / y_factor - inline_size;


 // Program Section //
//-----------------//

if(x_size > y_size) {
    resize(newsize=[max_size, x_factor * y_size, model_height]){
        if(flip_model == "yes") {
            mirror([0,1,0]) {
                rotate([180,180,0]) {    
                    create(outline, cut_polygons);
                }
            }
        } else {
            create(outline, cut_polygons);
        }
    }
} else {
    resize(newsize=[y_factor * x_size, max_size, model_height]){
        if(flip_model == "yes") {
            mirror([0,1,0]) {
                rotate([180,180,0]) {    
                    create(outline, cut_polygons);
                }
            }
        } else {
            create(outline, cut_polygons);
        }
    }
}


 // Module Section //
//----------------//

module create() {
    linear_extrude(height=1) {
        difference() {
            offset(r = -0) {
                offset(r = +outline_size) {
                    polygon(points = outline, convexity = 10);
                }
            }
            
            for(cut_polygon = [0:len(cut_polygons)]) {
                offset(r = +inline_edge_radius) {
                    offset(r = -inline_size - inline_edge_radius) {
                        polygon(points = cut_polygons[cut_polygon], convexity = 10);
                    }
                }
            }
        }
    }
}
